#!/bin/sh

### Tunable Parameters
#
# ROOT should point to the Cricket software package directory
#

ROOT="/home/aklmiu/research/project/Cricket"

#
# MODE - Run demo in simulation or normal mode
#

#MODE="normal"
MODE="simulation"

#
# COORDINATE_FILE points to the file that contains a set of 
# beacon coordinates.  This file must exist in one of the paths 
# in $CLASSPATH.
#

COORDINATE_FILE="beacons.conf"

#
# CRICKETD_IP_ADDR specifies the ip location of cricketd (aka gpsd).
# Use 127.0.0.1 for localhost.
#

CRICKETD_IP_ADDR=10.0.0.100
#CRICKETD_IP_ADDR=127.0.0.1

#
# Uncomment the following to enable raw CRICKETD output from CricketDaemon
# (Useful for debugging!)
#

CRICKETD_PRINT_ENABLE="-r"

#
# Beacons with fewer than ENOUGHSAMPLES within FILTER_WINDOW_MS window
# are ignored.  This is useful to filter invalid beacons as a result of
# noise generated by poor serial port connections or RF errors that
# corrupt the beacon information.  Usually, a value of 2-3 is enough for
# effective filtering. This filter has no effect on the accuracy of the 
# distance measurement between a beacon and a listener.
#

ENOUGHSAMPLES=2

#
# STDDEV_TOLERANCE_CM is a threshold value expressed in (cm)
# This value is used to define
#
# 1) The threshold for filtering outlier distance samples during the
#    tracking phase.  Currently, sample distances that are 
#    greater than 3*STDDEV_TOLERANCE_CM are filtered.  
#
# The recommended value for Cricket v1 is 15 and for Cricket v2 10 cm.
#

STDDEV_TOLERANCE_CM=10

#
# Size of history window in milliseconds. For Cricket v1, the recommended
# value is 15000.  For Cricket v2, the recommended value is 4000.  In all 
# cases, the mode value of the distances collected in the history window 
# is used to estimate the true distance between a beacon and the listener.
#

FILTER_WINDOW_MS=4000

#
# Uncommenting the following will enable Sun's Java HotSpot Server mode.
# The jvm will perform dynamic compiling to increase performance.
# However, the start up time will be quite slow.
# In general, the applications will run sluggishly in the first minute or
# so (depending on the computer speed) until the compilation is
# complete.  After the first minute, the CPU load should drop
# significantly.
#

JAVAHOTSPOT_ENABLE="-server"

#
#
############################################# End of Tunable Params

if [ "$OS" = "Windows_NT" ]
then SEP=";"
else SEP=":"
fi

NMSDEMOPATH="NMSDemo/2004/BeaconConfigDemo"
#BINPATH="bin"
BIN="${ROOT}/${NMSDEMOPATH}/"
LIB="${ROOT}/lib/"

CLASSPATH="${BIN}${SEP}${BIN}cricketdaemon.jar${SEP}${LIB}gnu.getopt.jar${SEP}${LIB}Jama-1.0.1.jar"

#echo $CLASSPATH

## Launch CricketDaemon
if [ "$MODE" = "simulation" ]
then 
echo ">>>>>>>>>>> Simulation Mode";
java $JAVAHOTSPOT -cp $CLASSPATH cricketdaemon.CricketDaemon -P \
-s$COORDINATE_FILE  -S$FILTER_WINDOW_MS -e$ENOUGHSAMPLES -t$STDDEV_TOLERANCE_MC \
-m$COORDINATE_FILE
else 
echo ">>>>>>>>>>> Normal Mode";
java $JAVAHOTSPOT -cp $CLASSPATH cricketdaemon.CricketDaemon -P \
-g$CRICKETD_IP_ADDR -S$FILTER_WINDOW_MS -e$ENOUGHSAMPLES -t$STDDEV_TOLERANCE_CM \
-m$COORDINATE_FILE $CRICKETD_PRINT
fi
