/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package navigator;

import java.util.*;
import java.awt.geom.*;

class QuadEdge
{
    public static final int VALID    = 0;
    public static final int INVALID  = 1;
    public static final int DOORJAMB = 2;

    String id;
    int type;
    
    Edge[] edges;

    public QuadEdge(String id, int type, String[] quadID, int[] quadEdgeID,
		    float[] x, float[] y)
    {
	this.id = id; 
	this.type = type; 
	
	edges = new Edge[4];
	for(int i = 0; i < 4; i++)
	    edges[i] = new Edge(this, i, quadID[i], quadEdgeID[i], x[i], y[i]);
	
    }

    public static QuadEdge getQuadEdge(Hashtable table, int id)
    {
	return (QuadEdge) table.get(Integer.toString(id));
    }

    public void insertTable(Hashtable edgeTable, Hashtable danglingTable)
    {
	Util.assert_nonjava(!edgeTable.containsKey(id), "Mesh::parseAnEdge()");
	edgeTable.put(id, this);
	
	Vector v;
	for(int i = 0; i < 4; i++) {
	    if(edges[i].org().isInvalid())
		continue;

	    QuadEdge e = (QuadEdge) edgeTable.get(edges[i].quadID);
	    if(e != null) // found QuadEdge
		edges[i].next = e.edges[edges[i].quadEdgeID];
	    else {
		v = (Vector) danglingTable.get(edges[i].quadID);
		if(v == null) {
		    v = new Vector();
		    danglingTable.put(edges[i].quadID, v);
		}
		else {
		    Util.assert_nonjava(!v.contains(this), 
				"QuadEdge::insertTable()");
		}
		v.add(this);
	    }
	}
	
	// resolve dangling edges that is missing this as reference
	v = (Vector) danglingTable.get(id);
	if(v != null) {
	    for(int i = 0; i < v.size(); i++) {
		QuadEdge e = (QuadEdge) v.get(i);
		e.linkMissingEdge(this);
	    }
	    danglingTable.remove(id);
	}
    }

    public void linkMissingEdge(QuadEdge e)
    {
	for(int i = 0; i < 4; i++) {
	    if(edges[i].quadID.equals(e.id))
		edges[i].next = e.edges[edges[i].quadEdgeID];
	}
    }

    public boolean equals(Object o)
    {
	return ((QuadEdge)o).id.equals(id);
    }
}
