/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.clientlib.data;

import cricketdaemon.clientlib.Associable;

/**
 * Contains information about a Beacon.  The unique id used for
 * Association is the string space+"-"+id.  The unique id is also used
 * for comparison between different BeaconRecords.
 *
 * @author Allen Miu
 */
public class BeaconRecord implements Associable, Comparable, Cloneable
{
    public String uniqueName;
    public String space;
    public Integer id;
    public Long lastUpdate;
    public DistanceStat distStat;
    public Position pos;

    public Object getUniqueIdentifier() { return uniqueName; }
    public boolean isAssociated(Associable o)
    {
	Object id = o.getUniqueIdentifier();
	if(!(id instanceof String))
	    return false;

	return id.equals(getUniqueIdentifier());
    }

    // facilitate sorting by ascending name order
    public int compareTo(Object o) 
    {
	return uniqueName.compareTo(((BeaconRecord)o).uniqueName);
    }
    
    public boolean equals(Object o)
    {
	if(!(o instanceof BeaconRecord))
	    return false;
	
	boolean result = compareTo(o) == 0;
	return result;
    }

    public void setUniqueName() { uniqueName = space+"-"+id; }
    
    public String toString()
    {
	String result = uniqueName+" "+lastUpdate+" "+distStat+" "+pos;
	return result;
    }

    public double getDist()
    {
	return distStat.dist;
    }

    // shallow clone
    public Object clone()
    {
	BeaconRecord result = new BeaconRecord();
	result.uniqueName = uniqueName;
	result.space = space;
	result.id = id;
	result.lastUpdate = lastUpdate;
	result.distStat = distStat;
	result.pos = pos;
	return result;
    }
}
