/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within a stat type.
 *
 * @author Allen Miu
 */
class StatParseHandlerMap extends ParseHandlerMap
{
    public static StatParseHandlerMap instance = 
	new StatParseHandlerMap();

    protected StatParseHandlerMap()
    {
	super();
	register("dist", "check_stat", "parse_dist" );
	register("median", "check_stat", "parse_median" );
	register("mean", "check_stat", "parse_mean" );
	register("mode", "check_stat", "parse_mode" );
	register("max", "check_stat", "parse_max" );
	register("min", "check_stat", "parse_min" );
	register("stddev", "check_stat", "parse_stddev" );
	register("var", "check_stat", "parse_var" );
	register("array:sample", "check_stat", "parse_asample");
    }

    public boolean check_stat(Object o)
    {
	return o instanceof DistanceStat;
    }
	
    public void parse_dist(Object dataStructure, String value)
	throws IOException 
    {
	//System.out.println("parse_dist "+value);
	DistanceStat a = (DistanceStat) dataStructure;
	a.dist = Util.parseDouble(value);
	//	System.out.println("parse_dist finish");
    }

    public void parse_median(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.median = Util.parseDouble(value);
    }
	
    public void parse_mean(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.mean = Util.parseDouble(value);
    }
	
    public void parse_mode(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.mode = Util.parseDouble(value);
    }
    
    public void parse_max(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.max = Util.parseDouble(value);
    }

    public void parse_min(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.min = Util.parseDouble(value);
    }
	
    public void parse_stddev(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.stddev = Util.parseDouble(value);
    }

    public void parse_var(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.variance = Util.parseDouble(value);
    }

    public void parse_asample(Object dataStructure, String value)
	throws IOException
    {
	DistanceStat a = (DistanceStat) dataStructure;

	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(a.samples,
			     sr, 
			     SampleArrayParseHandlerMap.instance);
    }

}

