/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.orient;


import cricketdaemon.module.*;
import cricketdaemon.position.*;
import cricketdaemon.space.DistanceEstimate;
import cricketdaemon.reader.CartesianCoord;
import cricketdaemon.reader.PhaseArrayRecord;
import cricketdaemon.reader.parse.ParseMap;
import cricketdaemon.reader.Beacon;
import cricketdaemon.UnitConv;
import cricketdaemon.Util;
import java.util.*;

/**
 * A module that computes the Orientation angle with respect to an
 * anchor beacon.
 *
 * FIXME: currently, it reports the absolute angle reported by SimReader
 *
 * Probably should use some averaged angle value and take hints from
 * BeaconStatModule.
 *
 * @author Allen Miu
 */
public class OrientModule extends ModuleThread 
    implements 
	Beacon.BeaconHandler,
	PositionData.PositionDataHandler
{
    /* OrientModule field types */
    public static final int ANGLE    = 1;
    public static final int ANCHOR   = 2;

    public OrientModule(ModuleThread mt, ModuleThread pt)
    {
	super("OrientModule", 0);
	BitSet desiredTypeMask = new BitSet();
	desiredTypeMask.set(ParseMap.ORIENT);     // for simulation demos
	desiredTypeMask.set(ParseMap.PHASEARRAY); // for Kevin
	mt.register(this, desiredTypeMask);

	if(pt != null) {
	    desiredTypeMask.clear();
	    desiredTypeMask.set(PositionModule.CARTESIAN_COORD);
	    pt.register(this, desiredTypeMask);
	}
    }

    protected void moduleCleanUp() { return; }

    public void processBeacon(Beacon newData, BitSet fieldMask)
    {
	Float orient;
	
	if(fieldMask.get(ParseMap.ORIENT)) {
	    orient = (Float) newData.getField(ParseMap.ORIENT);
	    OrientData newProcessedData = new OrientData(newData, orient);
	    BitSet orientFieldMask = (BitSet) CLEARMASK.clone();

	    // output - backward compatible
	    orientFieldMask.set(ANGLE);
	    transfer(newProcessedData, orientFieldMask);
	}

	if(fieldMask.get(ParseMap.PHASEARRAY)) {
	    ArrayList pa = (ArrayList) newData.getField(ParseMap.PHASEARRAY);
	    if(pa != null) {
		orient = processPhaseArray(pa);
		if(orient != null) {
		    OrientData newProcessedData = new OrientData(newData, orient);
		    BitSet orientFieldMask = (BitSet) CLEARMASK.clone();
		    orientFieldMask.set(ANGLE);
		    transfer(newProcessedData, orientFieldMask);
		}
	    }
	}
    }

    private Float processPhaseArray(ArrayList pa)
    {
	// kevin's algorithm goes here
	if(pa.size() > 0) {
	    PhaseArrayRecord r = (PhaseArrayRecord) pa.get(0);
	    //System.out.println("debug phase=" + r.orient + " " +r.sepDist);
	}
	return null;
    }
    
    public void processPositionData(PositionData newData, BitSet fieldMask)
    {
	if(fieldMask.get(PositionModule.CARTESIAN_COORD)) {
	    // array of beacon information containing 
	    // 1) the computed distance (and stats) between the listener 
	    //    and beacon
	    // 2) the beacon's space, id, and coordinates
	    // The length of the returned array represents the number of 
	    // beacons used/ in computing the listener's position
	    DistanceEstimate[] des = newData.getDistances();

	    /*
	    System.out.println("debug beacon="+des[0].space+"  coord="+
			       des[0].coord+" dist="+
			       UnitConv.cricketDistToCM(des[0].distance));
	    */

	    // Listener's position
	    CartesianCoord listenerPos = newData.getCoord();

	    // kevin's algorithm goes here
	    
	}
    }
}

// *TODO: change position/PositionData to include beacon coordinates
// *TODO: modify positionModule so that it filters beacons that do not have beacon coordinates
// *TODO: implement Kevin-specific input data types
// TODO: implement Kevin-specific output data types

