/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader.parse;

import cricketdaemon.Util;
import cricketdaemon.CricketDaemon;

/**
 * A FieldParser that parses the input bytes as an Integer.
 *
 * @author Allen Miu
 */
public class FieldParser_Integer implements FieldParser
{
    public static FieldParser instance = new FieldParser_Integer();

    private FieldParser_Integer()
    {
    }

    // Byte order: LSB ... MSB
    /*
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(length < 1 || length > 4)
	    Util.warn("FieldParser_Integer: length is invalid "+length);

	int result = normalize(beacon[offset]);
	
	if(length > 1)
	    result += normalize(beacon[offset+1]) * 256;      // 2^8
	if(length > 2)
	    result += normalize(beacon[offset+2]) * 65536;    // 2^16
	if(length > 3)
	    result += normalize(beacon[offset+3]) * 16777216; // 2^24
	
	//System.out.println("FieldParser_Integer "+result);

	return new Integer(result);
    }
    */
    // MSB ... LSB
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(CricketDaemon.listenerVersion < CricketDaemon.CRICKETV2) {
	    if(length < 1 || length > 4)
		Util.warn("FieldParser_Integer: length is invalid "+length);
	    
	    int result = normalize(beacon[offset]);
	    
	    if(length > 1)
		result = normalize(beacon[offset+1]) + result*256;
	    if(length > 2)
		result = normalize(beacon[offset+2]) + result*256;
	    if(length > 3)
		result = normalize(beacon[offset+3]) + result*256;
	    
	    //System.out.println("FieldParser_Integer "+result);
	    
	    return new Integer(result);
	}
	else {
	    return Integer.valueOf(new String(beacon, offset, length));
	}

    }

    // WARNING: assumes positive integer only !!!!
    public byte[] unparse(Object o)
    {
	int i = ((Integer) o).intValue();

	if(CricketDaemon.listenerVersion < CricketDaemon.CRICKETV2) {
	    byte[] result = new byte[4];
	    
	    for(int c = 3; c >= 0; c--) {
		result[c] = (byte) (i % 256);
		i -= normalize(result[c]);
		i /= 256;  // right shift
		if( i <= 0 )
		    break;
	    }
	    
	    return result;
	}
	else {
	    return String.valueOf(i).getBytes();
	}
    }

    public Object stringToReturnType(String s)
    {
	Object result = null;
	
	try {
	    result = Integer.valueOf(s);
	}
	catch(NumberFormatException e) {
	    // silently ignores;
	}
	
	return result;
    }

    public Object GPSDStringToReturnType(String s)
    {
	Object result = null;
	
	try {
	    if(CricketDaemon.listenerVersion < CricketDaemon.CRICKETV2) {
		result = Integer.valueOf(s, 16); // expects hex
	    }
	    else {
		result = Integer.valueOf(s); // expects decimal
	    }
	}
	catch(NumberFormatException e) {
	    // silently ignores;
	}
	
	return result;
    }

    private int normalize(byte b) {
	if( b < 0 )
	    return b + 256;
	else
	    return b;
    }

    public static void main(String[] args) {
	CricketDaemon.listenerVersion = CricketDaemon.CRICKETV2;
	FieldParser p = FieldParser_Integer.instance;
	Integer v = new Integer(0x12345678);
	byte[] r = p.unparse(v);
	//byte[] r = p.unparse(new Integer(152));
	for(int i = 0; i < r.length; i++)
	    System.out.println(i+" "+Integer.toHexString(r[i]));

	Integer parsed = (Integer) p.parse(r, 0, r.length);
	//System.out.println(Integer.toHexString(parsed.intValue()));
	System.out.print("correct: "+v+"\ntested:  ");
	System.out.println(Integer.toString(parsed.intValue()));
    }
}
